"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var md5 = require('md5');
const Banco_1 = __importDefault(require("../modelo/Banco"));
const Cargo_1 = __importDefault(require("./Cargo"));
class Funcionario {
    constructor() {
        this._idFuncionario = 0;
        this._nome = "";
        this._email = "";
        this._senha = "";
        this._recebeValeTransporte = 0;
        this._cargo = new Cargo_1.default();
    }
    create() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                const nome = this._nome;
                const email = this._email;
                const senha = md5(this._senha);
                const recebeValeTransporte = this._recebeValeTransporte;
                const Cargo_idCargo = this._cargo.idCargo;
                const PARAMETROS = [nome, email, senha, recebeValeTransporte, Cargo_idCargo];
                let SQL = "INSERT INTO funcionario (nome, email,senha, recebeValeTransporte, Cargo_idCargo) VALUES (?, ?, ?, ?, ?);";
                Banco_1.default.getConexao().query(SQL, PARAMETROS).then(([linhasBanco]) => {
                    resolve(linhasBanco);
                }).catch((erro) => {
                    reject(erro);
                });
            });
        });
    }
    read() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                const PARAMETROS = [''];
                const SQL = "SELECT idFuncionario,nome,email,recebeValeTransporte,idCargo,nomeCargo FROM funcionario JOIN cargo ON cargo.idCargo = funcionario.Cargo_idCargo  ORDER BY nome, nomeCargo";
                Banco_1.default.getConexao().query(SQL, PARAMETROS).then(([linhasBanco, fields]) => {
                    // console.log(rows);
                    resolve(linhasBanco);
                }).catch((erro) => {
                    reject(erro);
                });
            });
        });
    }
    getDadosGrafico() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                const PARAMETROS = [''];
                const SQL = "SELECT nomeCargo, count(nomecargo) AS qtdFuncionarios, sum(recebeValeTransporte) as valeTransporte FROM funcionario JOIN cargo ON idCargo= Cargo_idCargo GROUP BY cargo_idCargo ORDER BY nomeCargo;";
                Banco_1.default.getConexao().query(SQL, PARAMETROS).then(([linhasBanco, fields]) => {
                    // console.log(rows);
                    resolve(linhasBanco);
                }).catch((erro) => {
                    reject(erro);
                });
            });
        });
    }
    update() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                const nome = this._nome;
                const email = this._email;
                const senha = md5(this._senha);
                const recebeValeTransporte = this._recebeValeTransporte;
                const Cargo_idCargo = this._cargo.idCargo;
                const idFuncionario = this._idFuncionario;
                const PARAMETROS = [nome, email, senha, recebeValeTransporte, Cargo_idCargo, idFuncionario];
                const SQL = "update funcionario set nome=?,email=?,senha=?,recebeValeTransporte=?,Cargo_idCargo =? where idFuncionario = ?";
                Banco_1.default.getConexao().query(SQL, PARAMETROS).then(([linhasBanco, fields]) => {
                    //console.log(rows);
                    return resolve(linhasBanco);
                }).catch((erro) => {
                    return reject(erro);
                });
            });
        });
    }
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                const idFuncionario = this._idFuncionario;
                let PARAMETROS = [idFuncionario];
                let SQL = "delete from funcionario where idFuncionario = ?";
                Banco_1.default.getConexao().query(SQL, PARAMETROS).then(([linhasBanco, fields]) => {
                    return resolve(linhasBanco);
                }).catch((erro) => {
                    return reject(erro);
                });
            });
        });
    }
    login() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                const email = this._email;
                const senha = md5(this._senha);
                let PARAMETROS = [email, senha];
                let SQL = "SELECT COUNT(*) AS qtd ,nome,email,nomeCargo FROM funcionario JOIN cargo ON cargo.idCargo = funcionario.Cargo_idCargo  WHERE email =? AND senha =?;";
                Banco_1.default.getConexao().query(SQL, PARAMETROS).then(([linhasBanco, fields]) => {
                    resolve(linhasBanco);
                }).catch((erro) => {
                    reject(erro);
                });
            });
        });
    }
    get idFuncionario() {
        return this._idFuncionario;
    }
    set idFuncionario(idFuncionario) {
        this._idFuncionario = idFuncionario;
    }
    get nome() {
        return this._nome;
    }
    set nome(nome) {
        this._nome = nome;
    }
    get email() {
        return this._email;
    }
    set email(email) {
        this._email = email;
    }
    get senha() {
        return this._senha;
    }
    set senha(senha) {
        this._senha = senha;
    }
    get valeTransporte() {
        return this._recebeValeTransporte;
    }
    set valeTransporte(recebe) {
        this._recebeValeTransporte = recebe;
    }
    get cargo() {
        return this._cargo;
    }
    set cargo(cargo) {
        this._cargo = cargo;
    }
}
exports.default = Funcionario;
