var md5 = require('md5');
import * as Mysql from 'mysql2/promise';
import Banco from "../modelo/Banco";
import Cargo from "./Cargo";

export default class Funcionario {
    private _idFuncionario: number;
    private _nome: string;
    private _email: string;
    private _senha: string;
    private _recebeValeTransporte: number
    private _cargo: Cargo;

    constructor() {
        this._idFuncionario = 0;
        this._nome = "";
        this._email = "";
        this._senha = "";
        this._recebeValeTransporte = 0;
        this._cargo = new Cargo();
    }


    public async create(): Promise<Mysql.RowDataPacket[]> {
        return new Promise<Mysql.RowDataPacket[]>((resolve, reject) => {
            const nome = this._nome;
            const email = this._email;
            const senha = md5(this._senha);
            const recebeValeTransporte = this._recebeValeTransporte;
            const Cargo_idCargo = this._cargo.idCargo;

            const PARAMETROS = [nome, email, senha, recebeValeTransporte, Cargo_idCargo];

            let SQL = "INSERT INTO funcionario (nome, email,senha, recebeValeTransporte, Cargo_idCargo) VALUES (?, ?, ?, ?, ?);";

            Banco.getConexao().query<Mysql.RowDataPacket[]>(SQL, PARAMETROS).then(([linhasBanco]) => {
                resolve(linhasBanco)
            }).catch((erro) => {
                reject(erro);
            });
        });

    }

    public async read(): Promise<Mysql.RowDataPacket[]> {

        return new Promise<Mysql.RowDataPacket[]>((resolve, reject) => {
        
            const PARAMETROS = [''];
            const SQL = "SELECT idFuncionario,nome,email,recebeValeTransporte,idCargo,nomeCargo FROM funcionario JOIN cargo ON cargo.idCargo = funcionario.Cargo_idCargo  ORDER BY nome, nomeCargo";
            
            Banco.getConexao().query<Mysql.RowDataPacket[]>(SQL, PARAMETROS).then(([linhasBanco, fields]) => {
                // console.log(rows);
                resolve(linhasBanco)
            }).catch((erro) => {
                reject(erro);
            });
        });
    }
        public async getDadosGrafico(): Promise<Mysql.RowDataPacket[]> {

        return new Promise<Mysql.RowDataPacket[]>((resolve, reject) => {
        
            const PARAMETROS = [''];
            const SQL = "SELECT nomeCargo, count(nomecargo) AS qtdFuncionarios, sum(recebeValeTransporte) as valeTransporte FROM funcionario JOIN cargo ON idCargo= Cargo_idCargo GROUP BY cargo_idCargo ORDER BY nomeCargo;";
            
            Banco.getConexao().query<Mysql.RowDataPacket[]>(SQL, PARAMETROS).then(([linhasBanco, fields]) => {
                // console.log(rows);
                resolve(linhasBanco)
            }).catch((erro) => {
                reject(erro);
            });
        });
    }
    
    public async update(): Promise<Mysql.RowDataPacket[]> {
        return new Promise<Mysql.RowDataPacket[]>((resolve, reject) => {
            const nome = this._nome;
            const email = this._email;
            const senha = md5(this._senha);
            const recebeValeTransporte = this._recebeValeTransporte;
            const Cargo_idCargo = this._cargo.idCargo;
            const idFuncionario = this._idFuncionario;

            const PARAMETROS = [nome, email, senha, recebeValeTransporte, Cargo_idCargo,idFuncionario];
            const SQL = "update funcionario set nome=?,email=?,senha=?,recebeValeTransporte=?,Cargo_idCargo =? where idFuncionario = ?";
            Banco.getConexao().query<Mysql.RowDataPacket[]>(SQL, PARAMETROS).then(([linhasBanco, fields]) => {
                //console.log(rows);
                return resolve(linhasBanco)
            }).catch((erro) => {
                return reject(erro);
            });
        });

    }

    public async delete(): Promise<Mysql.RowDataPacket[]> {
        return new Promise<Mysql.RowDataPacket[]>((resolve, reject) => {
            const idFuncionario = this._idFuncionario;
            let PARAMETROS = [idFuncionario];
            let SQL = "delete from funcionario where idFuncionario = ?";
            Banco.getConexao().query<Mysql.RowDataPacket[]>(SQL, PARAMETROS).then(([linhasBanco, fields]) => {
                return resolve(linhasBanco)
            }).catch((erro) => {
                return reject(erro);
            });
        });
    }

    public async  login():  Promise<Mysql.RowDataPacket[]> {
        return new Promise<Mysql.RowDataPacket[]>((resolve, reject) => {
            const email = this._email;
            const senha = md5(this._senha);
            let PARAMETROS = [email, senha];
            let SQL = "SELECT COUNT(*) AS qtd ,nome,email,nomeCargo FROM funcionario JOIN cargo ON cargo.idCargo = funcionario.Cargo_idCargo  WHERE email =? AND senha =?;";

            Banco.getConexao().query<Mysql.RowDataPacket[]>(SQL, PARAMETROS).then(([linhasBanco, fields]) => {
                              
                    resolve(linhasBanco);
                
            }).catch((erro) => {
                 reject(erro);
            });
      
        });
     
    }


    

    public get idFuncionario(): number {
        return this._idFuncionario;
    }

    public set idFuncionario(idFuncionario: number) {
        this._idFuncionario = idFuncionario;
    }
    public get nome(): string {
        return this._nome;
    }

    public set nome(nome: string) {
        this._nome = nome;
    }

    public get email(): string {
        return this._email;
    }

    public set email(email: string) {
        this._email = email;
    }

    public get senha(): string {
        return this._senha;
    }

    public set senha(senha: string) {
        this._senha = senha;
    }

     public get valeTransporte():number{
        return this._recebeValeTransporte;
     }

     public set valeTransporte(recebe:number){
        this._recebeValeTransporte=recebe;
     }

     public get cargo():Cargo{
        return this._cargo;
     }

     public set cargo(cargo:Cargo){
        this._cargo=cargo;
     }
}


